// 
// lat - ConnectDialog.cs
// Author: Loren Bandiera
// Copyright 2005-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Net.Sockets;
using Gtk;
using Novell.Directory.Ldap;

namespace lat
{	
	public class ConnectDialog : Gtk.Dialog
	{
		protected Gtk.ComboBox serverTypeComboBox;
		protected EncryptionType encryption;
	
		protected Gtk.Entry hostEntry;
		protected Gtk.Entry portEntry;
		protected Gtk.Entry ldapBaseEntry;
		protected Gtk.Entry userEntry;
		protected Gtk.Entry passEntry;
		protected Gtk.RadioButton noEncryptionRadioButton;
		protected Gtk.RadioButton sslRadioButton;
		protected Gtk.RadioButton tlsRadioButton;
		protected Gtk.CheckButton saveProfileButton;
		protected Gtk.Entry profileNameEntry;
		protected Gtk.HBox stHBox;
		protected Gtk.Image image1;

		
		public ConnectDialog()
		{
			Stetic.Gui.Build(this, typeof(lat.ConnectDialog));
			
			Gdk.Pixbuf pb = Gdk.Pixbuf.LoadFromResource ("x-directory-remote-server-48x48.png");
			image1.Pixbuf = pb;
			
			CreateCombo ();
			
			this.Resizable = false;
		}

		#region internal_methods

		void CreateCombo ()
		{
			serverTypeComboBox = ComboBox.NewText ();
			serverTypeComboBox.AppendText ("OpenLDAP");
			serverTypeComboBox.AppendText ("Microsoft Active Directory");
			serverTypeComboBox.AppendText ("Fedora Directory Server");
			serverTypeComboBox.AppendText ("Generic LDAP server");

			serverTypeComboBox.Active = 0;
			serverTypeComboBox.Show ();

			stHBox.PackStart (serverTypeComboBox, true, true, 0);
		}
		
		string GetServerType ()
		{
			TreeIter iter;
			if (!serverTypeComboBox.GetActiveIter (out iter))
				return null;

			return (string) serverTypeComboBox.Model.GetValue (iter, 0);		
		}
		
		#endregion

		protected virtual void OnEncryptionToggled(object sender, System.EventArgs e)
		{
			if (tlsRadioButton.Active) {			
				portEntry.Text = "389";
				encryption = EncryptionType.TLS;
			} else if (sslRadioButton.Active) {
				portEntry.Text = "636";
				encryption = EncryptionType.SSL;
			} else {
				portEntry.Text = "389";
				encryption = EncryptionType.None;
			}
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			string serverType = GetServerType ();
			
			ConnectionData cd = new ConnectionData ();
			cd.Host = hostEntry.Text;
			cd.Port = int.Parse (portEntry.Text);
			cd.UserName = userEntry.Text;
			cd.Pass = passEntry.Text;
			cd.DirectoryRoot = ldapBaseEntry.Text;
			cd.ServerType = Util.GetServerType (serverType);			
			cd.DontSavePassword = false;
			cd.Encryption = encryption;
			
			if (saveProfileButton.Active) {
				cd.Name = profileNameEntry.Text;
				cd.Dynamic = false;
			} else {
				cd.Name = String.Format ("{0}:{1}", cd.Host, cd.Port);
				cd.Dynamic = true;
			}
			
			Connection conn = new Connection (cd);
			Global.Connections [cd.Name] = conn;			
		}


	}
}
