// 
// lat - GtkLdapBrowser.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Text.RegularExpressions;
using Gdk;
using GLib;
using Gtk;
using Novell.Directory.Ldap;
using Novell.Directory.Ldap.Utilclass;

namespace lat
{	
	public class GtkLdapBrowser : Gtk.Bin
	{
		protected TreeStore browserStore;
		protected TreeIter rootIter;
		
		protected Connection conn;
		
		protected ToolButton newButton;
		protected ToolButton deleteButton;
		
		protected bool isSingle;
		protected bool handlersSet;
		protected int browserSelectionMethod;
	
		enum TreeCols { Icon, DN, RDN };
	
		static TargetEntry[] dndSourceTable = new TargetEntry[]
		{
			new TargetEntry ("text/plain", 0, 1),
		};
	
		static TargetEntry[] dndTargetsTable = new TargetEntry[]
		{
			new TargetEntry ("text/uri-list", 0, 0),
			new TargetEntry ("text/plain", 0, 1),
		};	
	
		protected Gtk.TreeView treeview1;

		
		public GtkLdapBrowser()
		{
			Stetic.Gui.Build(this, typeof(lat.GtkLdapBrowser));
			
			Init ();

			Pixbuf dirIcon = Pixbuf.LoadFromResource ("x-directory-remote-server.png");					
			foreach (string n in Global.Connections.ConnectionNames) {
				TreeIter iter = browserStore.AppendValues (rootIter, dirIcon, n, n);
				browserStore.AppendValues (iter, null, "", "");				
			}
			
			TreePath path = browserStore.GetPath (rootIter);
			treeview1.ExpandRow (path, false);
		}

		public GtkLdapBrowser(Connection connection)
		{
			Stetic.Gui.Build(this, typeof(lat.GtkLdapBrowser));
			
			Init ();
			
			conn = connection;
			isSingle = true;
			
			if (!CheckConnection())
				return;

			Pixbuf dirIcon = Pixbuf.LoadFromResource ("x-directory-remote-server.png");			
			TreeIter iter = browserStore.AppendValues (rootIter, dirIcon, conn.Settings.Name, conn.Settings.Name);
			browserStore.AppendValues (iter, null, "", "");				

			TreePath path = browserStore.GetPath (rootIter);
			treeview1.ExpandRow (path, false);
		}
		
		public void AddConnection (string connectionName)
		{
			Pixbuf dirIcon = Pixbuf.LoadFromResource ("x-directory-remote-server.png");
			TreeIter iter = browserStore.AppendValues (rootIter, dirIcon, connectionName, connectionName);
			browserStore.AppendValues (iter, null, "");
		}
		
		public string GetActiveServerName ()
		{
			TreeModel model;
			TreeIter iter;

			if (treeview1.Selection.GetSelected (out model, out iter))
				return FindServerName (iter, model);
			
			return null;
		}
		
		public void GetSelectedEntry (out string dn, out string connection)
		{
			TreeModel model;
			TreeIter iter;

			if (treeview1.Selection.GetSelected (out model, out iter)) {
				string name = (string) browserStore.GetValue (iter, (int)TreeCols.DN);
				string connectionName =  FindServerName (iter, model);
						
				dn = name;
				connection = connectionName;
				
				return;
			}
		
			dn = null;
			connection = null;			
		}

		public TreeIter GetSelectedIter ()
		{
			TreeModel model;
			TreeIter iter;
				
			treeview1.Selection.GetSelected (out model, out iter);
			return iter;
		}
		
		public void Refresh ()
		{
			browserStore.Clear ();
			
			Gdk.Pixbuf dirIcon = Pixbuf.LoadFromResource ("x-directory-remote-server.png");
			rootIter = browserStore.AppendValues (dirIcon, "Servers", "Servers");
			TreePath path = browserStore.GetPath (rootIter);
			
			if (isSingle) {
			
				TreeIter iter = browserStore.AppendValues (rootIter, dirIcon, conn.Settings.Name, conn.Settings.Name);
				browserStore.AppendValues (iter, null, "", "");				
			
			} else {
			
				foreach (string n in Global.Connections.ConnectionNames) {
					TreeIter iter = browserStore.AppendValues (rootIter, dirIcon, n, n);
					browserStore.AppendValues (iter, null, "", "");
				}
			}

			treeview1.ExpandRow (path, false);	
		}
		
		public void RemoveRow (TreeIter iter)
		{
			browserStore.Remove (ref iter);
		}
		
		public void RemoveToolbarHandlers ()
		{
			if (handlersSet) {
				newButton.Clicked -= OnNewEntryActivate;
				deleteButton.Clicked -= OnDeleteActivate;

				handlersSet = false;
			}
		}
		
		public void SetToolbarHandlers (ToolButton newButton, ToolButton deleteButton)
		{
			this.newButton = newButton;
			this.deleteButton = deleteButton;
			
			this.newButton.Clicked += OnNewEntryActivate;
			this.deleteButton.Clicked += OnDeleteActivate;

			handlersSet = true;
		}
		
		public int BrowserSelectionMethod
		{
			get { return browserSelectionMethod; }
			set { browserSelectionMethod = value; }
		}

		public Connection CurrentConnection
		{
			set {
			
				conn = value;
				isSingle = true;
			
				if (!CheckConnection())
					return;

				browserStore.Clear ();
				
				Pixbuf dirIcon = Pixbuf.LoadFromResource ("x-directory-remote-server.png");
				rootIter = browserStore.AppendValues (dirIcon, "Servers", "Servers");
				TreeIter iter = browserStore.AppendValues (rootIter, dirIcon, value.Settings.Name, value.Settings.Name);
				browserStore.AppendValues (iter, null, "", "");				

				TreePath path = browserStore.GetPath (rootIter);
				treeview1.ExpandRow (path, false);			
			}
		}

		public string SelectedEntry
		{
			get {
				TreeModel model;
				TreeIter iter;
				
				if (treeview1.Selection.GetSelected (out model, out iter))
					return (string) browserStore.GetValue (iter, (int)TreeCols.DN);
				else
					return null;
			}
		}

		public string SelectedServer
		{
			get {

				TreeModel model;
				TreeIter iter;
				
				if (treeview1.Selection.GetSelected (out model, out iter))
					return FindServerName (iter, model);
				else			
					return null; 
			}
		}

		public event LdapEntrySelectedHandler EntrySelected;
			
		#region internal_methods

		void AddEntry (string name, Connection conn, TreeIter iter)
		{		
			try {

				Pixbuf pb = Pixbuf.LoadFromResource ("x-directory-normal.png");
		 		LdapEntry[] ldapEntries = conn.Data.GetEntryChildren (name); 

				foreach (LdapEntry le in ldapEntries) {

					Log.Debug ("\tchild: {0}", le.DN);
					DN dn = new DN (le.DN);
					RDN rdn = (RDN) dn.RDNs[0];

					TreeIter newChild;

					newChild = browserStore.AppendValues (iter, pb, le.DN, rdn.Value);
					browserStore.AppendValues (newChild, pb, "", "");
				}

			} catch (Exception e) {

				Log.Debug (e);

				string	msg = Mono.Unix.Catalog.GetString (
					"Unable to read data from server");

				Util.ShowErrorDialog ("Network error", msg);
			}
		
		}

		bool CheckConnection ()
		{
			try {
			
				if (!conn.IsConnected)
					conn.Connect ();
			
				return true;
				
			} catch (Exception e) { 

				Log.Debug (e);
				Util.ShowErrorDialog ("Connection error", e.Message);
			}
			
			return false;
		}
		
		void DispatchEntrySelectedEvent (string dn, bool host, string serverName)
		{
			if (EntrySelected != null)
				EntrySelected (this, new LdapEntrySelectedEventArgs (dn, host, serverName));
		}
		
		void DoPopUp()
		{
			Menu popup = new Menu();

			ImageMenuItem newItem = new ImageMenuItem (Stock.New, new Gtk.AccelGroup(IntPtr.Zero));
			newItem.Activated += new EventHandler (OnNewEntryActivate);
			newItem.Show ();
			popup.Append (newItem);

			MenuItem renameItem = new MenuItem ("Rename...");
			renameItem.Activated += new EventHandler (OnRenameActivate);
			renameItem.Show ();

			popup.Append (renameItem);

			MenuItem exportItem = new MenuItem ("Export...");
			exportItem.Activated += new EventHandler (OnExportActivate);
			exportItem.Show ();

			popup.Append (exportItem);

			ImageMenuItem deleteItem = new ImageMenuItem (Stock.Delete, new Gtk.AccelGroup(IntPtr.Zero));
			deleteItem.Activated += new EventHandler (OnDeleteActivate);
			deleteItem.Show ();

			popup.Append (deleteItem);
			popup.Popup(null, null, null, 3, Gtk.Global.CurrentEventTime);
		}

		string FindServerName (TreeIter iter, TreeModel model)
		{
			TreeIter parent;
			browserStore.IterParent (out parent, iter);
			
			if (!browserStore.IterIsValid (parent))
				return null;

			string parentName = (string)model.GetValue (parent, (int)TreeCols.DN);			
			if (parentName == "Servers")
				return (string)model.GetValue (iter, (int)TreeCols.DN);
			
			return FindServerName (parent, model);
		}

		void Init ()
		{
			browserStore = new TreeStore (typeof (Gdk.Pixbuf), typeof (string), typeof (string));
			
			treeview1.Model = browserStore;
			treeview1.HeadersVisible = false;
			
			treeview1.AppendColumn ("icon", new CellRendererPixbuf (), "pixbuf", (int)TreeCols.Icon);
			
			TreeViewColumn col;
			col = treeview1.AppendColumn ("DN", new CellRendererText (), "text", (int)TreeCols.DN);
			col.Visible = false;
			
			treeview1.AppendColumn ("RDN", new CellRendererText (), "text", (int)TreeCols.RDN);
	
			treeview1.ButtonPressEvent += OnRightClick;
			treeview1.Selection.Changed += OnSelectionChanged;

			Pixbuf dirIcon = Pixbuf.LoadFromResource ("x-directory-remote-server.png");
			rootIter = browserStore.AppendValues (dirIcon, "Servers", "Servers");

			Gtk.Drag.DestSet (this, DestDefaults.All, dndTargetsTable, Gdk.DragAction.Copy);
			Gtk.Drag.SourceSet (this, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask, dndSourceTable, Gdk.DragAction.Copy | DragAction.Move);
		}
		
		#endregion
		
		public void OnDeleteActivate (object o, EventArgs args) 
		{
			TreeModel model;
			TreeIter iter;

			if (!treeview1.Selection.GetSelected (out model, out iter))
				return;

			string dn = (string) browserStore.GetValue (iter, (int)TreeCols.DN);
			string serverName = FindServerName (iter, model);
			if (serverName == null)
				return;
			
			if (!isSingle)
				conn = Global.Connections [serverName];

			if (dn == conn.Settings.Host)
				return;

			try {
				if (Util.DeleteEntry (conn, dn))
					browserStore.Remove (ref iter);
			}
			catch {}
		}
		
		protected void OnDragBegin (object o, DragBeginArgs args)
		{
			Gdk.Pixbuf pb = Pixbuf.LoadFromResource ("text-x-generic.png");
			Gtk.Drag.SetIconPixbuf (args.Context, pb, 0, 0);
		}

		protected void OnDragDataGet (object o, DragDataGetArgs args)
		{
			Log.Debug ("BEGIN OnDragDataGet");

			Gtk.TreeModel model;
			Gtk.TreeIter iter;

			if (!treeview1.Selection.GetSelected (out model, out iter))
				return;

			string dn = (string) model.GetValue (iter, (int)TreeCols.DN);
			string data = null;

			Log.Debug ("Exporting entry: {0}", dn);

			if (!isSingle) {
				string serverName = FindServerName (iter, model);
				if (serverName == null)
					return;
					
				conn = Global.Connections [serverName];
			}

			Util.ExportData (conn, dn, out data);

			Atom[] targets = args.Context.Targets;
			args.SelectionData.Set (targets[0], 8, System.Text.Encoding.UTF8.GetBytes (data));

			Log.Debug ("END OnDragDataGet");
		}
		
		protected void OnDragDataReceived (object o, DragDataReceivedArgs args)
		{
			Log.Debug ("BEGIN OnDragDataReceived");

			bool success = false;
			string data = System.Text.Encoding.UTF8.GetString (args.SelectionData.Data);

			Gtk.TreeModel model;
			Gtk.TreeIter iter;

			if (!treeview1.Selection.GetSelected (out model, out iter))
				return;

			if (!isSingle) {
				string serverName = FindServerName (iter, model);
				if (serverName == null)
					return;
					
				conn = Global.Connections [serverName];
			}

			switch (args.Info) {
				
			case 0:
			{
				string[] uri_list = Regex.Split (data, "\r\n");

				Util.ImportData (conn, null, uri_list);
			
				success = true;
				break;
			}

			case 1:
				Util.ImportData (conn, null, data);
				success = true;
				break;

			}

			Log.Debug ("import success: {0}", success.ToString());

			Gtk.Drag.Finish (args.Context, success, false, args.Time);

			Log.Debug ("END OnDragDataReceived");
		}
		
		void OnExportActivate (object o, EventArgs args) 
		{
			string dn = this.SelectedEntry;			
			if (dn.Equals (null))
				return;

			TreeIter iter = GetSelectedIter ();
			string serverName = FindServerName (iter, browserStore);
			if (serverName == null)
				return;
		
			if (!isSingle)
				conn = Global.Connections [serverName];
				
			Util.ExportData (conn, null, dn);
		}
		
		public void OnNewEntryActivate (object o, EventArgs args) 
		{
			string dn = this.SelectedEntry;
			
			TreeModel model;
			TreeIter iter;

			if (treeview1.Selection.GetSelected (out model, out iter)) {
			
				string serverName = FindServerName (iter, model);
				if (serverName == null)
					return;

				if (!isSingle)
					conn = Global.Connections [serverName];
					
				CreateEntryHowtoDialog dlg = new CreateEntryHowtoDialog (conn, dn);
				dlg.Run ();
				dlg.Destroy ();
			}
		}
		
		void OnRenameActivate (object o, EventArgs args) 
		{
			string dn = this.SelectedEntry;
			TreeIter iter = GetSelectedIter ();

			string serverName = FindServerName (iter, browserStore);
			if (serverName == null)
				return;
		
			if (!isSingle)
				conn = Global.Connections [serverName];

			if (dn == conn.Settings.Host)
				return;

			RenameEntryDialog red = new RenameEntryDialog (conn, dn);
			red.Run ();

			TreeModel model;
			TreeIter iter2, parentIter;

			if (red.RenameHappened) {
				if (treeview1.Selection.GetSelected (out model, out iter2)) {
					browserStore.IterParent (out parentIter, iter2);
					TreePath tp = browserStore.GetPath (parentIter);
					treeview1.CollapseRow (tp);								
					treeview1.ExpandRow (tp, false);
				}
			}
			
			red.Destroy ();
		}
		
		[ConnectBefore]
		void OnRightClick (object o, ButtonPressEventArgs args)
		{
			if (args.Event.Button == 3)
				DoPopUp ();
		}
		
		protected void OnRowActivated (object o, RowActivatedArgs args)
		{
			if (this.BrowserSelectionMethod == 1)
				return;
		
			TreePath path = args.Path;
			TreeIter iter;
			
			if (browserStore.GetIter (out iter, path)) {

				string name = (string) browserStore.GetValue (iter, (int)TreeCols.DN);
				if (name == "Servers")
					return;

				string serverName = FindServerName (iter, browserStore);
				
				if (name.Equals (serverName)) {
					DispatchEntrySelectedEvent (name, true, serverName);
					return;
				}

				DispatchEntrySelectedEvent (name, false, serverName);
			} 		
		}
		
		protected void OnRowCollapsed (object o, RowCollapsedArgs args)
		{
			string name = (string) browserStore.GetValue (args.Iter, (int)TreeCols.DN);
			string serverName = FindServerName (args.Iter, browserStore);

			if (name == serverName) 
				return;

			Log.Debug ("collapsed row: {0}", name);

			TreeIter child;
			browserStore.IterChildren (out child, args.Iter);

			TreeIter lastChild = child;

			while (browserStore.IterNext (ref child)) {
				browserStore.Remove (ref lastChild);
				lastChild = child;
			}

			browserStore.Remove (ref lastChild);

			Gdk.Pixbuf pb = this.RenderIcon (Stock.Open, IconSize.Menu, "");
			browserStore.AppendValues (args.Iter, pb, "");
		}
		
		protected void OnRowExpanded (object o, RowExpandedArgs args)
		{		
			string name = (string) browserStore.GetValue (args.Iter, (int)TreeCols.DN);			
			if (name == "Servers")
				return;

			TreeIter child;
			browserStore.IterChildren (out child, args.Iter);
			
			string childName = (string)browserStore.GetValue (child, (int)TreeCols.DN);
			if (childName != "")
				return;			
					
			browserStore.Remove (ref child);
					
			Log.Debug ("Row expanded {0}", name);

			string serverName = FindServerName (args.Iter, browserStore);
			if (!isSingle)				
				conn = Global.Connections [serverName];

			if (!CheckConnection ())
				browserStore.AppendValues (args.Iter, null, "", "");
				
			if (name == serverName) {			
			
				Pixbuf pb = Pixbuf.LoadFromResource ("x-directory-remote-server.png");
				TreeIter i = browserStore.AppendValues (args.Iter, pb, conn.DirectoryRoot, conn.DirectoryRoot);
				browserStore.AppendValues (i, null, "", "");
				
			} else {

				AddEntry (name, conn, args.Iter);
				
			}
			
			TreePath path = browserStore.GetPath (args.Iter);
			treeview1.ExpandRow (path, false);				
		}
		
		void OnSelectionChanged (object o, EventArgs args)
		{
			if (this.BrowserSelectionMethod == 2)
				return;

			Gtk.TreeIter iter;
			Gtk.TreeModel model;
			
			if (treeview1.Selection.GetSelected (out model, out iter))  {
					
				string dn = (string) model.GetValue (iter, (int)TreeCols.DN);				
				string serverName = FindServerName (iter, model);
				
				if (dn.Equals (serverName)) {
					DispatchEntrySelectedEvent (dn, true, serverName);
					return;
				}

				DispatchEntrySelectedEvent (dn, false, serverName);
			}
		}
	}
	
	public delegate void LdapEntrySelectedHandler (object o, LdapEntrySelectedEventArgs args);
	
	public class LdapEntrySelectedEventArgs : EventArgs
	{
		public string DN;
		public bool IsHost;
		public string Server;
		
		public LdapEntrySelectedEventArgs (string DN, bool isHost, string serverName) : base ()
		{
			this.DN = DN;
			this.IsHost = isHost;
			this.Server = serverName;
		}
	}
}
