// 
// lat - GtkLdapObjectClassSchema.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using Gtk;
using Novell.Directory.Ldap;
using Novell.Directory.Ldap.Rfc2251;
using Novell.Directory.Ldap.Utilclass;

namespace lat
{	
	public class GtkLdapObjectClassSchema : Gtk.Bin
	{
		ListStore objRequiredStore;
		ListStore objOptionalStore;
	
		protected Gtk.TextView objNameTextview;
		protected Gtk.Entry objDescriptionEntry;
		protected Gtk.Entry objIDEntry;
		protected Gtk.TextView objSuperiorTextview;
		protected Gtk.CheckButton objObsoleteCheckbutton;
		protected Gtk.TreeView objRequiredTreeview;
		protected Gtk.TreeView objOptionalTreeview;
		protected Gtk.VPaned vpaned1;

		
		public GtkLdapObjectClassSchema()
		{
			Stetic.Gui.Build(this, typeof(lat.GtkLdapObjectClassSchema));
			
			objRequiredStore = new ListStore (typeof (string));
			objRequiredTreeview.Model = objRequiredStore;

			objOptionalStore = new ListStore (typeof (string));
			objOptionalTreeview.Model = objOptionalStore;

			objRequiredTreeview.AppendColumn ("Required Attributes", new CellRendererText (), "text", 0);
			objOptionalTreeview.AppendColumn ("Optional Attributes", new CellRendererText (), "text", 0);

			vpaned1.Position = 150;			
		}
		
		public void Clear ()
		{
			objNameTextview.Buffer.Text = "";
			objDescriptionEntry.Text = "";
			objIDEntry.Text = "";
			objObsoleteCheckbutton.Active = false;		
		}
		
		public void Show (SchemaParser sp)
		{
			try {
			
				objRequiredStore.Clear ();
				objOptionalStore.Clear ();

				objIDEntry.Text = sp.ID;
				objDescriptionEntry.Text = sp.Description;

				string tmp = "";
				foreach (string a in sp.Names)
					tmp += String.Format ("{0}\n", a);

				objNameTextview.Buffer.Text = tmp;

				tmp = "";
				foreach (string b in sp.Superiors)
					tmp += String.Format ("{0}\n", b);

				objSuperiorTextview.Buffer.Text = tmp;

				foreach (string c in sp.Required)
					objRequiredStore.AppendValues (c);

				foreach (string d in sp.Optional)
					objOptionalStore.AppendValues (d);

				objObsoleteCheckbutton.Active = sp.Obsolete;				
			
			} catch (Exception e) {
				Log.Debug (e);
			}
		}
	}
}
