// 
// lat - GtkLdapSchemaBrowser.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using Gtk;
using Novell.Directory.Ldap;
using Novell.Directory.Ldap.Utilclass;

namespace lat
{	
	public class GtkLdapSchemaBrowser : Gtk.Bin
	{
		protected TreeStore schemaStore;
		protected TreeIter rootIter;

		enum TreeCols { Icon, ObjectName };
		
		protected Gtk.TreeView treeview2;

		
		public GtkLdapSchemaBrowser()
		{
			Stetic.Gui.Build(this, typeof(lat.GtkLdapSchemaBrowser));
			
			schemaStore = new TreeStore (typeof (Gdk.Pixbuf), typeof (string));
			treeview2.Model = schemaStore;
			treeview2.HeadersVisible = false;
			treeview2.AppendColumn ("icon", new CellRendererPixbuf (), "pixbuf", (int)TreeCols.Icon);
			treeview2.AppendColumn ("ldapRoot", new CellRendererText (), "text", (int)TreeCols.ObjectName);
					
			Populate ();			
		}
		
		public void AddConnection (string name)
		{
			Gdk.Pixbuf dirIcon = Gdk.Pixbuf.LoadFromResource ("x-directory-remote-server.png");
			TreeIter iter = schemaStore.AppendValues (rootIter, dirIcon, name);
			schemaStore.AppendValues (iter, null, "");
		}
		
		public string GetActiveServerName ()
		{
			TreeModel model;
			TreeIter iter;

			if (treeview2.Selection.GetSelected (out model, out iter))
				return FindServerName (iter, model);
			
			return null;
		}
		
		public void Refresh ()
		{
			schemaStore.Clear ();
			Populate ();		
		}
		
		public event LdapSchemaSelectedHandler EntrySelected;
		
		#region internal_methods
			
		void DispatchEntrySelectedEvent (string name, string parent, string serverName)
		{
			if (EntrySelected != null)
				EntrySelected (this, new LdapSchemaSelectedEventArgs (name, parent, serverName));
		}
		
		string FindServerName (TreeIter iter, TreeModel model)
		{
			TreeIter parent;
			schemaStore.IterParent (out parent, iter);
			
			if (!schemaStore.IterIsValid (parent))
				return null;
			
			string parentName = (string)model.GetValue (parent, (int)TreeCols.ObjectName);			
			if (parentName == "Servers")
				return (string)model.GetValue (iter, (int)TreeCols.ObjectName);
			
			return FindServerName (parent, model);
		}
		
		string[] GetChildren (string parentName, Connection conn)
		{
			string[] childValues = null;
		
			switch (parentName) {
			
			case "Object Classes":
				childValues = conn.Data.ObjectClasses;
				break;
				
			case "Attribute Types":
				childValues = conn.Data.GetAttributeTypes ();
				break;
				
			case "Matching Rules":
				childValues = conn.Data.GetMatchingRules ();			
				break;
			
			case "LDAP Syntaxes":
				childValues = conn.Data.GetLdapSyntaxes ();
				break;
				
			default:
				break;
			}
			
			return childValues;			
		}
		
		void Populate ()
		{
			Gdk.Pixbuf dirIcon = Gdk.Pixbuf.LoadFromResource ("x-directory-remote-server.png");
			Gdk.Pixbuf folderIcon = Gdk.Pixbuf.LoadFromResource ("x-directory-normal.png");

			rootIter = schemaStore.AppendValues (dirIcon, "Servers");
			foreach (string n in Global.Connections.ConnectionNames) {
				TreeIter iter = schemaStore.AppendValues (rootIter, dirIcon, n);
				TreeIter objIter, attrIter, matIter, synIter;
				
				objIter = schemaStore.AppendValues (iter, folderIcon, "Object Classes");
				schemaStore.AppendValues (objIter, null, "");
				
				attrIter = schemaStore.AppendValues (iter, folderIcon, "Attribute Types");
				schemaStore.AppendValues (attrIter, null, "");
				
				matIter = schemaStore.AppendValues (iter, folderIcon, "Matching Rules");
				schemaStore.AppendValues (matIter, null, "");
					
				synIter = schemaStore.AppendValues (iter, folderIcon, "LDAP Syntaxes");
				schemaStore.AppendValues (synIter, null, "");	
			}
			
			TreePath path = schemaStore.GetPath (rootIter);
			treeview2.ExpandRow (path, false);
		}
		
		#endregion
		
		protected void OnRowActivated (object o, RowActivatedArgs args)
		{	
			TreePath path = args.Path;
			TreeIter iter;
			
			if (schemaStore.GetIter (out iter, path)) {

				string name = (string) schemaStore.GetValue (iter, (int)TreeCols.ObjectName);

				TreeIter parent;
				schemaStore.IterParent (out parent, iter);
				
				string parentName = (string) schemaStore.GetValue (parent, (int)TreeCols.ObjectName);
				string serverName = FindServerName (iter, schemaStore);
				
				if (name.Equals (serverName)) {
					DispatchEntrySelectedEvent (name, parentName, serverName);
					return;
				}

				DispatchEntrySelectedEvent (name, parentName, serverName);
			} 		
		}
		
		protected void OnRowExpanded (object o, RowExpandedArgs args)
		{
			string name = (string) schemaStore.GetValue (args.Iter, (int)TreeCols.ObjectName);			
			if (name == "Servers")
				return;

			TreeIter child;
			schemaStore.IterChildren (out child, args.Iter);
			
			string childName = (string) schemaStore.GetValue (child, (int)TreeCols.ObjectName);
			if (childName != "")
				return;
					
			schemaStore.Remove (ref child);
					
			Log.Debug ("Row expanded {0}", name);
			
			string serverName = FindServerName (args.Iter, schemaStore);
			Connection conn = Global.Connections [serverName];

			try {

				Gdk.Pixbuf pb = Gdk.Pixbuf.LoadFromResource ("text-x-generic.png");
		 				 		
		 		string[] kids = GetChildren (name, conn);
				foreach (string s in kids) { 
					schemaStore.AppendValues (args.Iter, pb, s);
				}
				
				TreePath path = schemaStore.GetPath (args.Iter);
				treeview2.ExpandRow (path, false);				

			} catch (Exception e) {

				schemaStore.AppendValues (args.Iter, null, "");
				Log.Debug (e);				

				string	msg = Mono.Unix.Catalog.GetString (
					"Unable to read schema information from server");

				Util.ShowErrorDialog ("Server error", msg);
			}
		}
	}
	
	public delegate void LdapSchemaSelectedHandler (object o, LdapSchemaSelectedEventArgs args);
	
	public class LdapSchemaSelectedEventArgs : EventArgs
	{
		public string Name;
		public string Parent;
		public string Server;
		
		public LdapSchemaSelectedEventArgs (string name, string parent, string serverName) : base ()
		{
			this.Name = name;
			this.Parent = parent;
			this.Server = serverName;
		}
	}
}
