// 
// lat - SearchBuilderDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Collections.Generic;
using Gtk;

namespace lat
{	
	public class SearchBuilderDialog : Gtk.Dialog
	{
		ComboBox opComboBox;
		ComboBox firstCritCombo;		
		List<ComboBox> allCombos;
		int numCriteria = 0;
		Dictionary<string,SearchCriteria> critTable;
		LdapSearch ls;	

		static string[] ops = { "begins with", "ends with", "equals", "contains", "is present" };
		static string[] boolOps = { "", "AND", "OR" };
	
		protected Gtk.HBox opHBox;
		protected Gtk.VBox critVbox;
		protected Gtk.Entry attributeEntry;
		protected Gtk.Entry valueEntry;

		
		public SearchBuilderDialog()
		{
			Stetic.Gui.Build(this, typeof(lat.SearchBuilderDialog));
			
			allCombos = new List<ComboBox> ();
			critTable = new Dictionary<string,SearchCriteria> ();
			ls = new LdapSearch ();	
			
			opComboBox = CreateCombo (ops);
			opHBox.Add (opComboBox);
		}

		public string UserFilter
		{
			get { return ls.Filter; }
		}

		#region internal_methods

		void BuildFilter ()
		{
			string boolOp = "";

			foreach (string key in critTable.Keys) 
			{
				SearchCriteria sc = (SearchCriteria) critTable [key];

				TreeIter iter;							
				if (!sc.critCombo.GetActiveIter (out iter))
					return;

				string s = (string) sc.critCombo.Model.GetValue (iter, 0);
			
				ls.addCondition (sc.attrEntry.Text,	s,	sc.valEntry.Text);
				if (!sc.boolCombo.GetActiveIter (out iter))
					return;

				string bc = (string) sc.boolCombo.Model.GetValue (iter, 0);
		
				if (!bc.Equals (""))
					boolOp = bc;
			}

			ls.addBool (boolOp);
			ls.endBool ();
		}

		void ComboSetActive (ComboBox cb, string[] list, string name)
		{		
			int count = 0;

			foreach (string s in list) {
				if (s.Equals (name))
					cb.Active = count;
					
				count++;
			}
		}

		static ComboBox CreateCombo (string[] list)
		{		
			ComboBox retVal = ComboBox.NewText ();

			foreach (string s in list)
				retVal.AppendText (s);

			retVal.Active = 0;
			retVal.Show ();

			return retVal;
		}		
		
		void CreateCritRow (string attr, string op, string val)
		{
			numCriteria++;

			HBox hbox = new HBox (false, 0);
			critVbox.PackStart (hbox, true, true, 0);

			Gtk.Entry attrEntry = new Gtk.Entry ();
			attrEntry.Text = attr;
			attrEntry.Show ();
			hbox.PackStart (attrEntry, true, true, 5);

			VBox vbox75 = new VBox (false, 0);
			vbox75.Show ();
			hbox.PackStart (vbox75, true, true, 5);

			ComboBox critCombo = CreateCombo (ops);
			ComboSetActive (critCombo, ops, op);
			vbox75.PackStart (critCombo, false, true, 16);
			
			Gtk.Entry valEntry = new Gtk.Entry ();
			valEntry.Text = val;
			valEntry.Show ();
			hbox.PackStart (valEntry, true, true, 5);

			VBox vbox76 = new VBox (false, 0);
			vbox76.Show ();
			hbox.PackStart (vbox76, true, true, 5);

			ComboBox boolCombo = CreateCombo (boolOps);
			boolCombo.Sensitive = false;
			vbox76.PackStart (boolCombo, false, true, 16);

			if (numCriteria == 1) {
				firstCritCombo = boolCombo;
				firstCritCombo.Changed += new EventHandler (OnBoolChanged);
			} else if (numCriteria > 1) {
				allCombos.Add (boolCombo);
			}

			SearchCriteria sc = new SearchCriteria (hbox, attrEntry, critCombo, valEntry, boolCombo);
			
			string key = "row" + numCriteria.ToString ();
			critTable.Add (key, sc);

			if (numCriteria > 1)
				ToggleBoolCombo (numCriteria);

			critVbox.ShowAll ();
		}
		
		void ToggleBoolCombo (int row)
		{
			string prevKey = "row" +  (row - 1).ToString();

			SearchCriteria prevSC = (SearchCriteria) critTable [prevKey];

			prevSC.boolCombo.Sensitive = !prevSC.boolCombo.Sensitive;
		}
		
		#endregion

		void OnBoolChanged (object o, EventArgs args)
		{
			foreach (ComboBox c in allCombos) {
				if (c == null)
					continue;

				if (c.Sensitive) {

					TreeIter iter;						
					if (!firstCritCombo.GetActiveIter (out iter))
						return;

					string s = (string) firstCritCombo.Model.GetValue (iter, 0);
					ComboSetActive (c, boolOps, s);
				}
			}
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			if (!attributeEntry.Text.Equals ("")) {

				TreeIter iter;				
				if (!opComboBox.GetActiveIter (out iter))
					return;

				string s = (string) opComboBox.Model.GetValue (iter, 0);

				// simple search; only one criteria
				ls.addCondition (attributeEntry.Text, s, valueEntry.Text);

			} else {
				// complex search
				BuildFilter ();
			}
		}

		protected virtual void OnAddClicked(object sender, System.EventArgs e)
		{
			TreeIter iter;
			if (!opComboBox.GetActiveIter (out iter))
				return;

			string s = (string) opComboBox.Model.GetValue (iter, 0);

			CreateCritRow (attributeEntry.Text, s, valueEntry.Text);
			attributeEntry.Text = "";
			valueEntry.Text = "";				
		}

		protected virtual void OnRemoveClicked(object sender, System.EventArgs e)
		{
			string key = "row" + numCriteria.ToString ();

			SearchCriteria sc = (SearchCriteria) critTable [key];
			sc.hbox.Destroy ();
			sc.attrEntry.Destroy ();
			sc.critCombo.Destroy ();
			sc.valEntry.Destroy ();
			sc.boolCombo.Destroy ();

			critTable.Remove (key);

			if (numCriteria > 1)
				ToggleBoolCombo (numCriteria);

			numCriteria--;		
		}
	}
	
	public struct SearchCriteria
	{
		public HBox hbox;
		public Gtk.Entry attrEntry;
		public ComboBox critCombo;
		public Gtk.Entry valEntry;
		public ComboBox boolCombo;

		public SearchCriteria (HBox aHbox, Gtk.Entry attr, ComboBox op, Gtk.Entry val, ComboBox bc)
		{
			hbox = aHbox;
			attrEntry = attr;
			critCombo = op;
			valEntry = val;
			boolCombo = bc;
		}
	}	
}
