// 
// lat - TemplateEditorDialog.cs
// Author: Loren Bandiera
// Copyright 2005-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using System.Collections.Generic;
using Novell.Directory.Ldap;
using Novell.Directory.Ldap.Utilclass;

namespace lat
{	
	public class TemplateEditorDialog : Gtk.Dialog
	{
		List<string> objectClass;
		Template t = null;
		Connection conn;
		bool isEdit = false;
		
		ListStore objListStore;
		ListStore attrListStore;	
	
		protected Gtk.Entry nameEntry;
		protected Gtk.TreeView objTreeView;
		protected Gtk.TreeView attrTreeView;

		
		public TemplateEditorDialog(Connection connection)
		{
			Stetic.Gui.Build(this, typeof(lat.TemplateEditorDialog));
			
			conn = connection;
		
			Init ();
		}

		public TemplateEditorDialog(Connection connection, Template theTemplate)
		{
			Stetic.Gui.Build(this, typeof(lat.TemplateEditorDialog));
			
			conn = connection;
			isEdit = true;
			t = theTemplate;

			Init ();

			nameEntry.Text = t.Name;
			nameEntry.Sensitive = false;

			foreach (string s in t.Classes) {
				objListStore.AppendValues (s);
				objectClass.Add (s);
			}

			ShowAttributes ();			
		}

		public Template UserTemplate
		{
			get { return t; }
		}

		#region internal_methods

		void Init ()
		{
			objectClass = new List<string> ();
			
			SetupTreeViews ();
			
			this.Resize (640,480);
		}

		void SetupTreeViews ()
		{
			// Object class
			objListStore = new ListStore (typeof (string));
			objTreeView.Model = objListStore;
			
			TreeViewColumn col;
			col = objTreeView.AppendColumn ("Name", new CellRendererText (), "text", 0);
			col.SortColumnId = 0;

			objListStore.SetSortColumnId (0, SortType.Ascending);

			// Attributes
			attrListStore = new ListStore (typeof (string), typeof (string), typeof (string));
			attrTreeView.Model = attrListStore;
			
			col = attrTreeView.AppendColumn ("Name", new CellRendererText (), "text", 0);
			col.SortColumnId = 0;

			col = attrTreeView.AppendColumn ("Type", new CellRendererText (), "text", 1);
			col.SortColumnId = 1;

			CellRendererText cell = new CellRendererText ();
			cell.Editable = true;
			cell.Edited += new EditedHandler (OnAttributeEdit);

			col = attrTreeView.AppendColumn ("Default Value", cell, "text", 2);
			col.SortColumnId = 2;

			attrListStore.SetSortColumnId (0, SortType.Ascending);
		}		
		
		void ShowAttributes ()
		{
			attrListStore.Clear ();

			string[] required, optional;			
			conn.Data.GetAllAttributes (objectClass, out required, out optional);

			foreach (string s in required) {

				if (isEdit) {
					attrListStore.AppendValues (s, 
						"Required", 
						t.GetAttributeDefaultValue (s));
				} else {

					attrListStore.AppendValues (s, "Required", "");
				}
			}

			foreach (string s in optional) {

				if (isEdit) {
					attrListStore.AppendValues (s, 
						"Required", 
						t.GetAttributeDefaultValue (s));

				} else {

					attrListStore.AppendValues (s, "Optional", "");
				}
			}
		}
		
		#endregion

		void OnAttributeEdit (object o, EditedArgs args)
		{
			TreeIter iter;

			if (!attrListStore.GetIterFromString (out iter, args.Path))
				return;

			string oldText = (string) attrListStore.GetValue (iter, 2);

			if (oldText.Equals (args.NewText))
				return;
			
			attrListStore.SetValue (iter, 2, args.NewText);		
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			if (isEdit) {
				t.Name = nameEntry.Text;
				t.ClearAttributes ();
			} else {

				t = new Template (nameEntry.Text);
			}

			t.AddClass (objectClass);	

			foreach (object[] row in attrListStore) {
				string nam = (string) row[0];
				string val = (string) row[2];
				
				if (nam == null || val == null || val == "")
					continue;
					
				t.AddAttribute (nam, val);
			}		
		}

		protected virtual void OnObjAddClicked(object sender, System.EventArgs e)
		{
			AddObjectClassDialog dlg = new AddObjectClassDialog (conn);
			dlg.Run ();
			
			foreach (string s in dlg.ObjectClasses) {
				if (objectClass.Contains (s))
					continue;
	
				objectClass.Add (s);
				objListStore.AppendValues (s);			
			}

			dlg.Destroy();
			ShowAttributes ();
		}

		protected virtual void OnObjRemovedClicked(object sender, System.EventArgs e)
		{
			Gtk.TreeIter iter;
			Gtk.TreeModel model;
			
			if (objTreeView.Selection.GetSelected (out model, out iter))  {
				string objClass = (string) model.GetValue (iter, 0);
				objectClass.Remove (objClass);

				objListStore.Remove (ref iter);

				ShowAttributes ();
			}
		}

		protected virtual void OnObjClearClicked(object sender, System.EventArgs e)
		{
			objListStore.Clear ();
			objectClass.Clear ();

			attrListStore.Clear ();		
		}

		protected virtual void OnAttrRemoveClicked(object sender, System.EventArgs e)
		{
			Gtk.TreeIter iter;
			Gtk.TreeModel model;
			
			if (attrTreeView.Selection.GetSelected (out model, out iter)) 
				attrListStore.Remove (ref iter);
		}

		protected virtual void OnAttrClearClicked(object sender, System.EventArgs e)
		{
			attrListStore.Clear ();
		}
	}
}
