// 
// lat - TemplatesDialog.cs
// Author: Loren Bandiera
// Copyright 2005-2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using Gtk;
using Novell.Directory.Ldap;
using Novell.Directory.Ldap.Utilclass;

namespace lat
{	
	public class TemplatesDialog : Gtk.Dialog
	{
		ListStore store;
		Connection conn;
		
		protected Gtk.TreeView templateTreeView;

		
		public TemplatesDialog(Connection connection)
		{
			Stetic.Gui.Build(this, typeof(lat.TemplatesDialog));
			
			conn = connection;
			
			store = new ListStore (typeof (string));
			templateTreeView.Model = store;
			
			TreeViewColumn col;
			col = templateTreeView.AppendColumn ("Name", new CellRendererText (), "text", 0);
			col.SortColumnId = 0;

			store.SetSortColumnId (0, SortType.Ascending);
			
			ListTemplates ();
			
			this.Resize (320, 300);
		}

		void EditTemplate (string name)
		{
			Template t = Global.Templates.Lookup (name);
			TemplateEditorDialog ted = new TemplateEditorDialog (conn, t);
			ted.Run ();

			if (ted.UserTemplate == null)
				return;

			ListTemplates ();
			ted.Destroy ();
		}

		void ListTemplates ()
		{
			store.Clear ();

			string[] names = Global.Templates.GetTemplateNames ();
			foreach (string n in names)
				store.AppendValues (n);
		}

		protected virtual void OnAddClicked(object sender, System.EventArgs e)
		{
			TemplateEditorDialog ted = new TemplateEditorDialog (conn);
			ted.Run ();

			if (ted.UserTemplate == null)
				return;

			Global.Templates.Add (ted.UserTemplate);
			ListTemplates ();
			
			ted.Destroy ();
		}

		protected virtual void OnEditClicked(object sender, System.EventArgs e)
		{
			Gtk.TreeIter iter;
			Gtk.TreeModel model;
			
			if (templateTreeView.Selection.GetSelected (out model, out iter))  {
				string name = (string) model.GetValue (iter, 0);
				EditTemplate (name);
			}
		}

		protected virtual void OnDeleteClicked(object sender, System.EventArgs e)
		{
			Gtk.TreeIter iter;
			Gtk.TreeModel model;
			
			if (!templateTreeView.Selection.GetSelected (out model, out iter)) 
				return;

			string name = (string) model.GetValue (iter, 0);
			
			string tmp = String.Format (Mono.Unix.Catalog.GetString ("Are you sure you want to delete:"));
			string msg = String.Format ("{0}\n\n{1}", tmp, name);

			if (Util.AskYesNo (this, msg)) {
				Global.Templates.Delete (name);
				ListTemplates ();
			}		
		}

		protected virtual void OnRowActivated(object o, Gtk.RowActivatedArgs args)
		{
			TreePath path = args.Path;
			TreeIter iter;
			
			if (store.GetIter (out iter, path)) {
				string name = (string) store.GetValue (iter, 0);
				EditTemplate (name);
			} 		
		}
	}
}
