// 
// lat - NewAdUserViewDialog.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using Gtk;
using System;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Text;
using Mono.Security.Protocol.Ntlm;
using Novell.Directory.Ldap;
using lat;

namespace AD.Gui
{
	public class NewAdUserViewDialog : Gtk.Dialog
	{
		// ACCOUNT_DISABLE|NORMAL_ACCOUNT|DONT_EXPIRE_PASSWORD
		int userAC = 66050;
		string[] groupList;
		ComboBox primaryGroupComboBox;
		
		protected Connection conn;
		protected bool missingValues;
		protected bool errorOccured;
		protected string defaultNewContainer;
		
		protected Gtk.HBox groupHBox;
		protected Gtk.Label usernameLabel;
		protected Gtk.Label fullnameLabel;
		protected Gtk.Entry firstNameEntry;
		protected Gtk.Entry initialsEntry;
		protected Gtk.Entry lastNameEntry;
		protected Gtk.Entry displayNameEntry;
		protected Gtk.Entry upnEntry;
		protected Gtk.Entry usernameEntry;
		protected Gtk.Entry passwordEntry;
		protected Gtk.CheckButton mustChangePwdCheckButton;
		protected Gtk.CheckButton cantChangePwdCheckButton;
		protected Gtk.CheckButton pwdNeverExpiresCheckButton;
		protected Gtk.CheckButton accountDisabledCheckButton;

		
		public NewAdUserViewDialog(Connection connection, string newContainer)
		{
			Stetic.Gui.Build(this, typeof(AD.Gui.NewAdUserViewDialog));
			
			this.conn = connection;
			this.defaultNewContainer = newContainer;
			
			groupList = GetGroups ();

			CreateCombo ();

			this.Run ();

			while (missingValues || errorOccured) {

				if (missingValues)
					missingValues = false;
				else if (errorOccured)
					errorOccured = false;

				this.Run ();				
			}

			this.Destroy ();
		}

		#region internal_methods
		
		void CreateCombo ()
		{
			primaryGroupComboBox = ComboBox.NewText ();

			foreach (string n in groupList)
				primaryGroupComboBox.AppendText (n);

			primaryGroupComboBox.Active = 0;
			primaryGroupComboBox.Show ();

			// FIXME: primary group
			primaryGroupComboBox.Sensitive = false;

			groupHBox.PackStart (primaryGroupComboBox, true, true, 0);
		}
		
		LdapEntry CreateEntry (string dn)
		{
			LdapAttributeSet aset = new LdapAttributeSet();
			string fullName = String.Format ("{0} {1}", firstNameEntry.Text, lastNameEntry.Text);
			aset.Add (new LdapAttribute ("cn", fullName));
			aset.Add (new LdapAttribute ("gecos", fullName));
			aset.Add (new LdapAttribute ("objectClass", new string[] {"top", "person", "organizationalPerson","user"}));
			aset.Add (new LdapAttribute ("givenName", firstNameEntry.Text));
			aset.Add (new LdapAttribute ("sn", lastNameEntry.Text));
			aset.Add (new LdapAttribute ("userPrincipalName", upnEntry.Text));
			aset.Add (new LdapAttribute ("sAMAccountName", usernameEntry.Text));
			aset.Add (new LdapAttribute ("userAccountControl", userAC.ToString()));
			aset.Add (new LdapAttribute ("displayName", displayNameEntry.Text));
			aset.Add (new LdapAttribute ("initials", initialsEntry.Text));
			
			LdapEntry newEntry = new LdapEntry (dn, aset);
			return newEntry;
		}
		
		string[] GetGroups ()
		{
			LdapEntry[] grps = conn.Data.SearchByClass ("group");
			List<string> glist = new List<string> ();
	
			foreach (LdapEntry e in grps) {

				LdapAttribute nameAttr;
				nameAttr = e.getAttribute ("cn");

				glist.Add (nameAttr.StringValue);
			}

			return glist.ToArray ();
		}
		
		#endregion

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			LdapEntry entry = null;
			string userDN = null;
			
			if (this.defaultNewContainer == string.Empty) {
			
				SelectContainerDialog scd =	new SelectContainerDialog (conn);
				scd.Title = "Save Group";
				scd.Message = String.Format ("Where in the directory would\nyou like save the user\n{0}?", displayNameEntry.Text);
				scd.Run ();

				if (scd.DN == "")
					return;

				userDN = String.Format ("cn={0},{1}", displayNameEntry.Text, scd.DN);
				scd.Destroy ();
			
			} else {
			
				userDN = String.Format ("cn={0},{1}", displayNameEntry.Text, this.defaultNewContainer);
			}
			
			entry = CreateEntry (userDN);

			string[] missing = LdapEntryAnalyzer.CheckRequiredAttributes (conn, entry);
			if (missing.Length != 0) {
				Util.ShowMissingAttrbutesAlert (missing);
				missingValues = true;
				return;
			}

			if (!Util.AddEntry (conn, entry))
				errorOccured = true;
		}

		protected virtual void OnNameChanged(object sender, System.EventArgs e)
		{
			if (initialsEntry.Text != "")
				fullnameLabel.Text = String.Format ("{0} {1} {2}", firstNameEntry.Text, initialsEntry.Text, lastNameEntry.Text);
			else
				fullnameLabel.Text = String.Format ("{0} {1}", firstNameEntry.Text, lastNameEntry.Text);
				
			displayNameEntry.Text = fullnameLabel.Text;
			usernameLabel.Text = upnEntry.Text;
		}

		protected virtual void OnDisplayNameFocusIn(object o, Gtk.FocusInEventArgs args)
		{
			string suid = Util.SuggestUserName (
					firstNameEntry.Text, 
					lastNameEntry.Text);

			usernameEntry.Text = suid;

			if (displayNameEntry.Text != "")
				return;

			if (initialsEntry.Text.Equals("")) {

				displayNameEntry.Text = String.Format ("{0} {1}", 
					firstNameEntry.Text, 
					lastNameEntry.Text);

			} else {

				String format = "";
				if (initialsEntry.Text.EndsWith("."))
					format = "{0} {1} {2}";
				else
					format = "{0} {1}. {2}";

				displayNameEntry.Text = String.Format (format, 
					firstNameEntry.Text, 
					initialsEntry.Text, 
					lastNameEntry.Text);
			}		
		}
	}
}
