// 
// lat - JpegViewDialog.cs
// Author: Loren Bandiera
// Copyright 2006 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.IO;
using Gtk;

namespace Jpeg.Gui
{
	public class JpegViewDialog : Gtk.Dialog
	{
		byte[] rawBytes = null;
		
		protected Gtk.Entry filenameEntry;
		protected Gtk.Image jpegImage;

		
		public JpegViewDialog(byte[] imageData)
		{
			Stetic.Gui.Build(this, typeof(Jpeg.Gui.JpegViewDialog));
			
			if (imageData != null) {
				try {
					Gdk.Pixbuf pb = new Gdk.Pixbuf (imageData);
					jpegImage.Pixbuf = pb;
				} catch (Exception e) {
					lat.Log.Debug (e);
				}
			}
			
			this.Resize (300, 400);
		}

		public byte[] RawFileBytes
		{
			get { return rawBytes; }
		}

		static byte[] ReadFile (Stream stream)
		{
			byte[] buffer = new byte[32768];
			using (MemoryStream ms = new MemoryStream())
			{
				while (true)
				{
					int read = stream.Read (buffer, 0, buffer.Length);
					if (read <= 0)
						return ms.ToArray();
					ms.Write (buffer, 0, read);
				}
			}
		}

		protected virtual void OnOkClicked(object sender, System.EventArgs e)
		{
			if (filenameEntry.Text == null || filenameEntry.Text == "")
				return;
				
			try {
			
				FileStream fs = File.OpenRead(filenameEntry.Text);
				rawBytes = ReadFile (fs);				
				
			} catch (Exception ex) {
				lat.Log.Debug (ex);
			}		
		}

		protected virtual void OnBrowseClicked(object sender, System.EventArgs e)
		{
			FileChooserDialog fcd = new FileChooserDialog (
				"Choose an image",
				Gtk.Stock.Open, 
				null, 
				FileChooserAction.Open);

			fcd.AddButton (Gtk.Stock.Cancel, ResponseType.Cancel);
			fcd.AddButton (Gtk.Stock.Open, ResponseType.Ok);

			fcd.SelectMultiple = false;

			ResponseType response = (ResponseType) fcd.Run();
			if (response == ResponseType.Ok) {
				filenameEntry.Text = fcd.Filename;
				
				try {
					Gdk.Pixbuf pb = new Gdk.Pixbuf (fcd.Filename);
					jpegImage.Pixbuf = pb;
				} catch (Exception ex) {
					lat.Log.Debug (ex);
				}
			} 
		
			fcd.Destroy();		
		}
	}
}
